/*
 *	CrabDA.c;  a somewhat malignant Desk Accessory inspired by A.K.
 *	Dewdney's Computer Recreations column in Scientific American.
 *
 *  submitted by ** Carlos Weber 70277.726 **
 *
 *	changed for a Mac Plus & LightSpeedC by Galen C. Hunt
 *											777 South 400 East #32
 *											St. George, UT 84770
 */


#include <MacTypes.h>
#include <Quickdraw.h>
#include <DeskMgr.h>
#include <EventMgr.h>
#include <DeviceMgr.h>
#include <WindowMgr.h>
#include <FileMgr.h>

static GrafPtr scrPtr;
static BitMap	crabMap;
static Rect		wholeRect	= { 0,0,16,16 };
static Rect		topleftRect = { 0,0, 8, 8 };
static Rect		drawRect;
static int		crabCount, crabOn = 0;
static int		left, top;
static Point	crabWhere[40];

static unsigned long crabBits[4] =
{
	0xBEAA7755,
	0xAAAA7F55,	/* 0xBEAA7F55 */
	0xBEAA7F55,	/* 0xBEAA7F55 */
	0xAAAA5555
}; 


/* Functions */
int main(cntrlParam *, DCtlPtr, int);
void crabOpen(void);
void crabControl(void);
char *getA5(void);


char *getA5()
{
	asm
	{
		move.l	a5, d0
		return
	}
}

int main(theIo, dce, sel)
register cntrlParam *theIo;
register DCtlPtr dce;
register int sel;
{	
	switch (sel)
	{
		case 0:	/* open */
			if (crabOn)
				break;
			crabOn = 1;
			scrPtr = (GrafPtr)NewPtr((long)sizeof(GrafPort));
			crabOpen();
			dce->dCtlEMask = 0;
			dce->dCtlDelay = 1;	/* XXX 10 */
			dce->dCtlFlags |= dNeedTime;
			break;
		case 2:	/* control */
			crabControl();
			break;
		case 4:	/* close */
			DisposPtr(crabMap.baseAddr);
			ClosePort(scrPtr);
			DisposPtr(scrPtr);
			break;
	}
	return 0;
}

void crabOpen()		/* really our Open routine	*/
{
	register int i, j, yo;
	register char *wp;
	register Point *cp;
	GrafPtr tempPtr;
	
	/*
	 *	Set up bitmaps for the screen, the crab, and the gray stuff the
	 *	crabs leave behind.  The latter 2 bitmaps are bigger than they
	 *	really need to be because of the constraint that rowBytes must
	 *	be even.
	 */
	
	GetPort(&tempPtr);
	OpenPort(scrPtr);
	BlockMove(DeskPattern,scrPtr->bkPat, (long)sizeof(Pattern));
	left = scrPtr->portRect.right;
	top = scrPtr->portRect.bottom;
	
	crabMap.baseAddr = wp = NewPtr((long)sizeof(crabBits));
	BlockMove(crabBits, wp, (long)sizeof(crabBits));


	crabMap.rowBytes = 2;
	crabMap.bounds = wholeRect;
	
	/*
	 *	allocate a random # of crabs to starting positions down the
	 *	left edge of the screen, starting just below the menu bar
	 */
	
	cp = crabWhere;
	i = 40;
	j = 0;
	drawRect.left = 8;
	drawRect.right = 16;
	
	while (i--)
	{
		if (Random() & 1)
		{
			cp->h = 8;
			yo = cp->v = 20 + (i << 3);

			/*
			 *	paste the top left corner of the crab bitmap -- i.e., the
			 *	crab itself-- in each crab's starting position on the screen
			 */
			 
			drawRect.bottom = (drawRect.top = yo) + 8;
			CopyBits(&crabMap, &scrPtr->portBits, &topleftRect, &drawRect, srcCopy, 0);

			j++;
			cp++;
		}
	}
	crabCount = j;  	/* keep track of how many crabs were allocated */
	SetPort(tempPtr);
}
	

void crabControl()  /* the guts of the DA */
{
	register int j, xo, yo;
	register Point *cp;
	register int offset;
	GrafPtr tempPtr;


	GetPort(&tempPtr);
	SetPort(scrPtr);
	j = crabCount;
	cp = crabWhere;
	
	while (j--)
    {
		/*
		 *	first, replace the current crab with gray
		 */

		xo = cp->h;
		yo = cp->v;
		
		drawRect.bottom = (drawRect.top = yo) + 8;
		drawRect.right = (drawRect.left = xo) + 8;

		EraseRect(&drawRect);

		/*
		 *	change crab's position a little bit vertically, somewhat
		 *	more horizontally.
		 */
		offset = 2 * (Random() & 0x0003) + 2;
		xo += ((Random() & 0x0001) ? offset:left-offset);
		xo %= left;

		switch (Random() & 0x0003)
		{
			case 0:
				yo += top;
				yo -= 2;
				break;
			case 3:
				yo += 2;
				break;
		}
		yo %= top;

		/*
		 *	redraw the little buggers
		 */
		drawRect.bottom = (drawRect.top = yo) + 8;
		drawRect.right = (drawRect.left = xo) + 8;
		
		CopyBits(&crabMap,&scrPtr->portBits,&topleftRect,&drawRect,srcCopy,0);
		
		cp->h = xo;
		cp->v = yo;
		cp++;
	}
	SetPort(tempPtr);
}

/* - End of Crabs.c - */